package com.vmware.alarms;

import com.vmware.vim25.*;
import java.util.*;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;

/**
 *<pre>
 *VMPowerStateAlarm
 *
 *This sample which creates an Alarm to monitor the virtual machine's power state
 *
 *<b>Parameters:</b>
 *url         [required] : url of the web service
 *username    [required] : username for the authentication
 *password    [required] : password for the authentication
 *vmname      [required] : Name of the virtual machine
 *alarm       [required] : Name of the alarms
 *
 *<b>Command Line:</b>
 *Create an alarm AlarmABC on a virtual machine
 *run.bat com.vmware.vm.VMPowerStateAlarm --url [webserviceurl]
 *--username [username] --password  [password] --vmname [vmname] --alarm [alarm]
 *</pre>
 */


public class VMPowerStateAlarm {
   private static class TrustAllTrustManager1 implements javax.net.ssl.TrustManager,
                                                         javax.net.ssl.X509TrustManager {
      public java.security.cert.X509Certificate[] getAcceptedIssuers() {
         return null;
      }

      public boolean isServerTrusted(java.security.cert.X509Certificate[] certs) {
         return true;
      }

      public boolean isClientTrusted(java.security.cert.X509Certificate[] certs) {
         return true;
      }

      public void checkServerTrusted(java.security.cert.X509Certificate[] certs,
                                     String authType)
         throws java.security.cert.CertificateException {
         return;
      }

      public void checkClientTrusted(java.security.cert.X509Certificate[] certs,
                                     String authType)
         throws java.security.cert.CertificateException {
         return;
      }
   }

   private static final ManagedObjectReference SVC_INST_REF = new ManagedObjectReference();
   private static final String SVC_INST_NAME = "ServiceInstance";
   private static VimPortType vimPort;
   private static VimService vimService;
   private static ServiceContent serviceContent;
   private static Boolean isConnected = false;
   private static ManagedObjectReference rootRef = null;
   private static ManagedObjectReference propCollectorRef = null;
   private static ManagedObjectReference vmMor = null;
   private static ManagedObjectReference alarmManager = null;

   private static String url;
   private static String userName;
   private static String password;
   private static Boolean help = false;
   private static String alarm = null;
   private static String vmname = null;

   private static void trustAllHttpsCertificates()
   throws Exception {
      javax.net.ssl.TrustManager[] trustAllCerts = new javax.net.ssl.TrustManager[1];
      javax.net.ssl.TrustManager tm = new TrustAllTrustManager1();
      trustAllCerts[0] = tm;
      javax.net.ssl.SSLContext sc = javax.net.ssl.SSLContext.getInstance("SSL");
      javax.net.ssl.SSLSessionContext sslsc = sc.getServerSessionContext();
      sslsc.setSessionTimeout(0);
      sc.init(null, trustAllCerts, null);
      javax.net.ssl.HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
   }

   private static void getConnectionParameters(String[] args) {
      int ai = 0;
      String param = "";
      String val = "";
      while (ai < args.length) {
         param = args[ai].trim();
         if (ai + 1 < args.length) {
            val = args[ai + 1].trim();
         }
         if (param.equalsIgnoreCase("--help")) {
            help = true;
            break;
         } else if (param.equalsIgnoreCase("--url")
                       && !val.startsWith("--") && !val.isEmpty()) {
            url = val;
         } else if (param.equalsIgnoreCase("--username")
                       && !val.startsWith("--") && !val.isEmpty()) {
            userName = val;
         } else if (param.equalsIgnoreCase("--password")
                       && !val.startsWith("--") && !val.isEmpty()) {
            password = val;
         }
         val = "";
         ai += 2;
      }
      if(url == null || userName == null || password == null) {
         throw new IllegalArgumentException(
            "Expected --url, --username, --password arguments.");
      }
   }

   private static void getInputParameters(String[] args) {

      int ai = 0;
      String param = "";
      String val = "";
      while (ai < args.length) {
         param = args[ai].trim();
         if (ai + 1 < args.length) {
            val = args[ai + 1].trim();
         }
         if (param.equalsIgnoreCase("--vmname") && !val.startsWith("--") &&
               !val.isEmpty()) {
            vmname = val;
         }
         if (param.equalsIgnoreCase("--alarm") && !val.startsWith("--") &&
               !val.isEmpty()) {
            alarm = val;
         }
         val = "";
         ai += 2;
      }
      if (vmname == null || alarm == null) {
         throw new IllegalArgumentException(
         "Expected --vmname, --alarm arguments properly");
      }
   }

   private static void connect()
      throws Exception {
      HostnameVerifier hv = new HostnameVerifier() {
         public boolean verify(String urlHostName, SSLSession session) {
            return true;
         }
      };
      trustAllHttpsCertificates();
      HttpsURLConnection.setDefaultHostnameVerifier(hv);

      SVC_INST_REF.setType(SVC_INST_NAME);
      SVC_INST_REF.setValue(SVC_INST_NAME);

      vimService = new VimService();
      vimPort = vimService.getVimPort();
      Map<String, Object> ctxt =
         ((BindingProvider) vimPort).getRequestContext();

      ctxt.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, url);
      ctxt.put(BindingProvider.SESSION_MAINTAIN_PROPERTY, true);

      serviceContent = vimPort.retrieveServiceContent(SVC_INST_REF);
      vimPort.login(serviceContent.getSessionManager(),
            userName,
            password, null);
      isConnected = true;

      rootRef = serviceContent.getRootFolder();
      propCollectorRef = serviceContent.getPropertyCollector();
      alarmManager = serviceContent.getAlarmManager();
   }

   private static void disconnect()
      throws Exception {
      if (isConnected) {
         vimPort.logout(serviceContent.getSessionManager());
      }
      isConnected = false;
   }

   /**
    * Gets the vM traversal spec.
    *
    * @return TraversalSpec specification to get to the VirtualMachine managed object.
    */
   private static TraversalSpec getVMTraversalSpec() {
      // Create a traversal spec that starts from the 'root' objects
      // and traverses the inventory tree to get to the VirtualMachines.
      // Build the traversal specs bottoms up

      //Traversal to get to the VM in a VApp
      TraversalSpec vAppToVM = new TraversalSpec();
      vAppToVM.setName("vAppToVM");
      vAppToVM.setType("VirtualApp");
      vAppToVM.setPath("vm");
      //Traversal spec for VApp to VApp
      TraversalSpec vAppToVApp = new TraversalSpec();
      vAppToVApp.setName("vAppToVApp");
      vAppToVApp.setType("VirtualApp");
      vAppToVApp.setPath("resourcePool");
      //SelectionSpec for VApp to VApp recursion
      SelectionSpec vAppRecursion = new SelectionSpec();
      vAppRecursion.setName("vAppToVApp");
      //SelectionSpec to get to a VM in the VApp
      SelectionSpec vmInVApp = new SelectionSpec();
      vmInVApp.setName("vAppToVM");
      //SelectionSpec for both VApp to VApp and VApp to VM
      List<SelectionSpec> vAppToVMSS = new ArrayList<SelectionSpec>();
      vAppToVMSS.add(vAppRecursion);
      vAppToVMSS.add(vmInVApp);
      vAppToVApp.getSelectSet().addAll(vAppToVMSS);

      //This SelectionSpec is used for recursion for Folder recursion
      SelectionSpec sSpec = new SelectionSpec();
      sSpec.setName("VisitFolders");

      // Traversal to get to the vmFolder from DataCenter
      TraversalSpec dataCenterToVMFolder = new TraversalSpec();
      dataCenterToVMFolder.setName("DataCenterToVMFolder");
      dataCenterToVMFolder.setType("Datacenter");
      dataCenterToVMFolder.setPath("vmFolder");
      dataCenterToVMFolder.setSkip(false);
      dataCenterToVMFolder.getSelectSet().add(sSpec);

      // TraversalSpec to get to the DataCenter from rootFolder
      TraversalSpec traversalSpec = new TraversalSpec();
      traversalSpec.setName("VisitFolders");
      traversalSpec.setType("Folder");
      traversalSpec.setPath("childEntity");
      traversalSpec.setSkip(false);
      List<SelectionSpec> sSpecArr = new ArrayList<SelectionSpec>();
      sSpecArr.add(sSpec);
      sSpecArr.add(dataCenterToVMFolder);
      sSpecArr.add(vAppToVM);
      sSpecArr.add(vAppToVApp);
      traversalSpec.getSelectSet().addAll(sSpecArr);
      return traversalSpec;
   }

   /**
    * Uses the new RetrievePropertiesEx method to emulate the now deprecated RetrieveProperties method
    *
    * @param listpfs
    * @return list of object content
    * @throws Exception
    */
   private static List<ObjectContent> retrievePropertiesAllObjects(List<PropertyFilterSpec> listpfs)
      throws Exception {

      RetrieveOptions propObjectRetrieveOpts = new RetrieveOptions();

      List<ObjectContent> listobjcontent = new ArrayList<ObjectContent>();

      try {
         RetrieveResult rslts =
            vimPort.retrievePropertiesEx(propCollectorRef,
                                         listpfs,
                                         propObjectRetrieveOpts);
         if (rslts != null && rslts.getObjects() != null &&
               !rslts.getObjects().isEmpty()) {
            listobjcontent.addAll(rslts.getObjects());
         }
         String token = null;
         if(rslts != null && rslts.getToken() != null) {
            token = rslts.getToken();
         }
         while (token != null && !token.isEmpty()) {
            rslts = vimPort.continueRetrievePropertiesEx(propCollectorRef, token);
            token = null;
            if (rslts != null) {
               token = rslts.getToken();
               if (rslts.getObjects() != null && !rslts.getObjects().isEmpty()) {
                 listobjcontent.addAll(rslts.getObjects());
               }
            }
         }
      } catch (SOAPFaultException sfe) {
         printSoapFaultException(sfe);
      } catch (Exception e) {
         System.out.println(" : Failed Getting Contents");
         e.printStackTrace();
      }
      return listobjcontent;
   }

   /**
    * Get the MOR of the Virtual Machine by its name.
    * @param vmName The name of the Virtual Machine
    * @return The Managed Object reference for this VM
    */
   private static ManagedObjectReference getVmByVMname(String vmName) {
      ManagedObjectReference retVal = null;
      ManagedObjectReference rootFolder = serviceContent.getRootFolder();
      try {
         TraversalSpec tSpec = getVMTraversalSpec();
         // Create Property Spec
         PropertySpec propertySpec = new PropertySpec();
         propertySpec.setAll(Boolean.FALSE);
         propertySpec.getPathSet().add("name");
         propertySpec.setType("VirtualMachine");

         // Now create Object Spec
         ObjectSpec objectSpec = new ObjectSpec();
         objectSpec.setObj(rootFolder);
         objectSpec.setSkip(Boolean.TRUE);
         objectSpec.getSelectSet().add(tSpec);

         // Create PropertyFilterSpec using the PropertySpec and ObjectPec
         // created above.
         PropertyFilterSpec propertyFilterSpec = new PropertyFilterSpec();
         propertyFilterSpec.getPropSet().add(propertySpec);
         propertyFilterSpec.getObjectSet().add(objectSpec);

         List<PropertyFilterSpec> listpfs = new ArrayList<PropertyFilterSpec>(1);
         listpfs.add(propertyFilterSpec);

         List<ObjectContent> listobjcont = retrievePropertiesAllObjects(listpfs);

         if (listobjcont != null) {
            for (ObjectContent oc : listobjcont) {
               ManagedObjectReference mr = oc.getObj();
               String vmnm = null;
               List<DynamicProperty> dps = oc.getPropSet();
               if (dps != null) {
                  for (DynamicProperty dp : dps) {
                     vmnm = (String) dp.getVal();
                  }
               }
               if (vmnm != null && vmnm.equals(vmName)) {
                  retVal = mr;
                  break;
               }
            }
         }
      } catch (SOAPFaultException sfe) {
         printSoapFaultException(sfe);
      } catch (Exception e) {
         e.printStackTrace();
      }
      return retVal;
   }

   /**
    * Creates the state alarm expression.
    *
    * @return the state alarm expression
    * @throws Exception the exception
    */
   private static StateAlarmExpression createStateAlarmExpression()
      throws Exception {
      StateAlarmExpression expression = new StateAlarmExpression();
      expression.setType("VirtualMachine");
      expression.setStatePath("runtime.powerState");
      expression.setOperator(StateAlarmOperator.IS_EQUAL);
      expression.setRed("poweredOff");
      return expression;
   }

   /**
    * Creates the power on action.
    *
    * @return the method action
    */
   private static MethodAction createPowerOnAction() {
      MethodAction action = new MethodAction();
      action.setName("PowerOnVM_Task");
      MethodActionArgument argument = new MethodActionArgument();
      argument.setValue(null);
      action.getArgument().addAll(Arrays.asList(new MethodActionArgument[]{argument}));
      return action;
   }

   /**
    * Creates the alarm trigger action.
    *
    * @param methodAction the method action
    * @return the alarm triggering action
    * @throws Exception the exception
    */
   private static AlarmTriggeringAction createAlarmTriggerAction(MethodAction methodAction)
      throws Exception {
      AlarmTriggeringAction alarmAction = new AlarmTriggeringAction();
      alarmAction.setYellow2Red(true);
      alarmAction.setAction(methodAction);
      return alarmAction;
   }

   /**
    * Creates the alarm spec.
    *
    * @param action the action
    * @param expression the expression
    * @return the alarm spec object
    * @throws Exception the exception
    */
   private static AlarmSpec createAlarmSpec(AlarmAction action,
                                            AlarmExpression expression)
      throws Exception {
      AlarmSpec spec = new AlarmSpec();
      spec.setAction(action);
      spec.setExpression(expression);
      spec.setName(alarm);
      spec.setDescription("Monitor VM state and send email if VM power's off");
      spec.setEnabled(true);
      return spec;
   }

   /**
    * Creates the alarm.
    *
    * @param alarmSpec the alarm spec object
    * @throws Exception the exception
    */
   private static void createAlarm(AlarmSpec alarmSpec)
      throws Exception {
      ManagedObjectReference alarmmor
         = vimPort.createAlarm(alarmManager, vmMor, alarmSpec);
      System.out.println("Successfully created Alarm: " + alarmmor.getValue());
   }

   private static void printSoapFaultException(SOAPFaultException sfe) {
      System.out.println("SOAP Fault -");
      if (sfe.getFault().hasDetail()) {
         System.out.println(sfe.getFault().getDetail().getFirstChild().getLocalName());
      }
      if (sfe.getFault().getFaultString() != null) {
         System.out.println("\n Message: " + sfe.getFault().getFaultString());
      }
   }

   private static void printUsage() {
      System.out.println("This sample which creates "
         + "an Alarm to monitor the virtual machine's power state");
      System.out.println("\nParameters:");
      System.out.println("url         [required] : url of the web service");
      System.out.println("username    [required] : username for the authentication");
      System.out.println("password    [required] : password for the authentication");
      System.out.println("vmname      [required] : Name of the virtual machine");
      System.out.println("alarm       [required] : Name of the alarms");
      System.out.println("\nCommand:");
      System.out.println("Create an alarm AlarmABC on a virtual machine");
      System.out.println("run.bat com.vmware.vm.VMPowerStateAlarm --url [webserviceurl]");
      System.out.println("--username [username] --password  [password] --vmname [vmname]");
      System.out.println("--alarm [alarm]");
   }

   public static void main(String [] args) {
      try {
         getConnectionParameters(args);
         if(help) {
            printUsage();
            return;
         }
         getInputParameters(args);
         connect();
         vmMor = getVmByVMname(vmname);
         if (vmMor != null) {
            StateAlarmExpression expression = createStateAlarmExpression();
            MethodAction methodAction = createPowerOnAction();
            AlarmAction alarmAction
               = (AlarmAction) createAlarmTriggerAction(methodAction);
            AlarmSpec alarmSpec = createAlarmSpec(alarmAction, expression);
            createAlarm(alarmSpec);
         } else {
            System.out.println("Virtual Machine " + vmname + " Not Found");
         }
      } catch (IllegalArgumentException iae) {
         System.out.println(iae.getMessage());
         printUsage();
      } catch (SOAPFaultException sfe) {
         printSoapFaultException(sfe);
      } catch (Exception e) {
         System.out.println("Exception encountered " + e);
      } finally {
         try {
            disconnect();
         } catch (SOAPFaultException sfe) {
            printSoapFaultException(sfe);
         } catch (Exception e) {
            System.out.println("Failed to disconnect - " + e.getMessage());
            e.printStackTrace();
         }
      }
   }

}
